/*
	Relay -- a tool to record and play Quake2 demos
	Copyright (C) 2000 Conor Davis

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	Conor Davis
	cedavis@planetquake.com
*/

#ifndef __R_LOCAL_H
#define __R_LOCAL_H

#include <stdio.h>

#include "shared.h"
#include "block.h"
#include "dm2.h"
#include "endian.h"
#include "mem.h"
#include "pak.h"
#include "proxy.h"
#include "q2defines.h"
#include "q2utils.h"
#include "utils.h"

#define EDICT2NUM(x)	((int)((byte *)(x) - (byte *)ge->edicts) / ge->edict_size)
#define NUM2EDICT(x)	((edict_t *)((byte *)ge->edicts + ge->edict_size*(x)))

extern game_import_t	gi;
extern game_import_t	my_gi;

extern game_export_t	globals;
extern game_export_t	*ge;

extern cvar_t			*maxclients;
extern int				level_frame;

extern proxy_t			proxydata;

extern PFILE			*outfile;
extern dm2_t			dm2out;
extern byte 			areaportals[MAX_MAP_AREAPORTALS/8];

extern block_t			tempblock;
extern char 			tempblock_buffer[MAX_MSGLEN];
extern block_t			reliable;
extern char 			reliable_buffer[MAX_SVSLEN];
extern block_t			unreliable;
extern char 			unreliable_buffer[MAX_SVSLEN];

// the "gameversion" client command will print this plus compile date
#define GAMEVERSION "q2server"

#define FRAMETIME		0.1

// memory tags to allow dynamic memory to be cleaned up
#define TAG_GAME	765 	// clear when unloading the dll
#define TAG_LEVEL	766 	// clear when loading a new level

// r_export.c
extern void SpawnEntities (char *mapname, char *entities, char *spawnpoint);
extern void ClientThink (edict_t *ent, usercmd_t *cmd);
extern qboolean ClientConnect (edict_t *ent, char *userinfo);
extern void ClientUserinfoChanged (edict_t *ent, char *userinfo);
extern void ClientDisconnect (edict_t *ent);
extern void ClientBegin (edict_t *ent);
extern void ClientCommand (edict_t *ent);
extern void RunEntity (edict_t *ent);
extern void WriteGame (char *filename, qboolean autosave);
extern void ReadGame (char *filename);
extern void WriteLevel (char *filename);
extern void ReadLevel (char *filename);
extern void InitGame (void);
extern void G_RunFrame (void);
extern void ShutdownGame (void);
extern void ServerCommand (void);

// r_import.c
extern void import_bprintf(int printlevel, char *fmt, ...);
extern void import_cprintf(edict_t *ent, int printlevel, char *fmt, ...);
extern void import_centerprintf(edict_t *ent, char *fmt, ...);
extern void import_sound(edict_t *ent, int channel, int soundindex, float volume, float attenuation, float timeofs);
extern void import_positioned_sound(vec3_t origin, edict_t *ent, int channel, int soundindex, float volume, float attenuation, float timeofs);
extern void import_configstring(int num, char *string);
extern int import_modelindex(char *name);
extern int import_soundindex(char *name);
extern int import_imageindex(char *name);
extern void import_setmodel(edict_t *ent, char *name);
extern void import_SetAreaPortalState(int portalnum, qboolean connected);
extern void import_linkentity(edict_t *ent);
extern void import_unlinkentity(edict_t *ent);
extern void import_multicast(vec3_t origin, multicast_t to);
extern void import_unicast(edict_t *ent, qboolean isreliable);
extern void import_WriteChar(int c);
extern void import_WriteByte(int c);
extern void import_WriteShort(int c);
extern void import_WriteLong(int c);
extern void import_WriteFloat(float f);
extern void import_WriteString(char *s);
extern void import_WritePosition(vec3_t pos);
extern void import_WriteDir(vec3_t pos);
extern void import_WriteAngle(float f);

#endif	// __LOCAL_H
