/*
	Relay -- a tool to record and play Quake2 demos
	Copyright (C) 2000 Conor Davis

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	Conor Davis
	cedavis@planetquake.com
*/

#include <string.h>

#include "rp_local.h"
#include "dm2.h"
#include "q2utils.h"
#include "utils.h"

// generic menu functions
void UpdateLayout(edict_t *ent)
{
	if (ent->client->curmenu)
		Menu_Display(ent->client->curmenu, ent->client->layout, sizeof(ent->client->layout));
	else if (ent->client->player != -1 && ent->client->relayflags & RC_LAYOUT)
		strcpy(ent->client->layout, dm2in.players[ent->client->player].layout);
	else
		ent->client->layout[0] = 0;
	ent->client->layout_modified = true;
}

void OpenMenu(edict_t *ent, void (*Show)(menu_t *))
{
	Menu_Open(ent, &ent->client->curmenu, Show, "");
	UpdateLayout(ent);
}

void CloseMenu(edict_t *ent)
{
	Menu_Close(&ent->client->curmenu);
	UpdateLayout(ent);
}

void CloseAllMenus(edict_t *ent)
{
	Menu_CloseAll(&ent->client->curmenu);
	UpdateLayout(ent);
}

void UpdateAllMenus(int id)
{
	edict_t *ent;
	int 	i;

	for (i = 0; i < game.maxclients; i++)
	{
		ent = g_edicts + i + 1;
		if (!ent->inuse || !ent->client)
			continue;

		Menu_Update(&ent->client->curmenu, ent->client->layout, sizeof(ent->client->layout), id);
	}
}

void Select_OpenMenu(menu_t *menu, menuitem_t *item, int key)
{
	edict_t *ent;

	ent = menu->client;
	Menu_Open(ent, &ent->client->curmenu, 
		(void (*)(menu_t *))item->param[0],
		"");
	UpdateLayout(ent);
}

//
// Demo Menu
//

void DemoMenu_Show(menu_t *menu)
{
	char	buf[256];
	int 	i, count;

	Menu_Start(menu);
	if (!menu->title)
		menu->title = Z_Strdup("Demo Info");
	if (!menu->id)
		menu->id = MENU_DEMO;

	count = 0;
	for (i = 0; i < dm2in.maxclients; i++)
	{
		if (ISBITSET(current_connected, i))
			count++;
	}

	switch(dm2in.svd.isdemo)
	{
	case RECORD_NETWORK:	sprintf(buf, "Type:           Network"); break;
	case RECORD_CLIENT: 	sprintf(buf, "Type:           Client");  break;
	case RECORD_SERVER: 	sprintf(buf, "Type:           Server");  break;
	case RECORD_RELAY:		sprintf(buf, "Type:           Relay");	 break;
	default:				sprintf(buf, "Type:           Unknown"); break;
	}
	Menu_AddItem(buf, "");
	if (dm2in.svd.game[0])
		sprintf(buf, "Game:           %s", dm2in.svd.game);
	else
		strcpy(buf,  "Game:           baseq2");
	Menu_AddItem(buf, "");
	COM_FileBase(dm2in.configstrings[CS_MODELS+1], buf);
	Menu_AddItem(va("Map:            %s", buf), "");
	Menu_AddItem(va("Desc:           %.32s", dm2in.svd.mapname), "");
	Menu_AddItem(va("Maxclients:     %d", dm2in.maxclients), "");
	Menu_AddItem(va("Active players: %d", count), "");

	Menu_Finish(menu);
}

//
// Players Menu
//

void PlayersMenu_Select(menu_t *menu, menuitem_t *item, int key)
{
  edict_t	*ent;

  ent = (edict_t *)menu->client;
	if (ChangePlayer(ent, item->param[0]))
		gi.cprintf(ent, PRINT_HIGH, "Unable to switch to player %s\n", item->text);
}

void PlayersMenu_Show(menu_t *menu)
{
  edict_t		*ent;
	int 		i, num, index;
 
	ent = (edict_t *)menu->client;
	// if menu is newly opened (cur == -1), put cursor on currently
	// tracked player, otherwise try to keep the cursor on the same
	// player after the menu changes
	if (menu->cur == -1 || !menu->items)
		num = ent->client->player;
	else
		num = menu->items[menu->cur].param[0];

	Menu_Start(menu);
	if (!menu->title)
		menu->title = Z_Strdup("Players");
	if (!menu->id)
		menu->id = MENU_PLAYERS;

	Menu_AddItem("No player", "S1", PlayersMenu_Select, -1);
	if (dm2in.svd.isdemo == RECORD_RELAY)
	{
		for (i = 0; i < dm2in.maxclients; i++)
		{
			if (ISBITSET(current_connected, i))
			{
				index = Menu_AddItem(PlayerName(dm2in.configstrings[CS_PLAYERSKINS+i]), "S1", PlayersMenu_Select, i);
				if (i == num)
					menu->cur = index;
			}
		}
	}
	else
		Menu_AddItem(PlayerName(dm2in.configstrings[CS_PLAYERSKINS+dm2in.svd.player]), "S1", PlayersMenu_Select, 0);

	Menu_Finish(menu);
}

//
// Settings Menu
//

void SettingsMenu_Select(menu_t *menu, menuitem_t *item, int key)
{
	edict_t *ent;

	ent = (edict_t *)menu->client;
	ent->client->relayflags ^= item->param[0];
	if (item->param[0] & RC_STATUSBAR)
	{
		if (ent->client->relayflags & RC_STATUSBAR)
		{
			if (ent->client->player != -1)
			{
				gi.WriteByte(SVC_CONFIGSTRING);
				gi.WriteShort(CS_STATUSBAR);
				gi.WriteString(dm2in.configstrings[CS_STATUSBAR]);
				gi.unicast(ent, false);
			}
		}
		else
		{
			gi.WriteByte(SVC_CONFIGSTRING);
			gi.WriteShort(CS_STATUSBAR);
			gi.WriteString("");
			gi.unicast(ent, false);
		}
	}

	Menu_Update(&ent->client->curmenu, ent->client->layout, sizeof(ent->client->layout), MENU_SETTINGS);
}

void SettingsMenu_Show(menu_t *menu)
{
	edict_t *ent;
	int 	relayflags;

	ent = (edict_t *)menu->client;
	relayflags = ent->client->relayflags;

	Menu_Start(menu);
	if (!menu->title)
		menu->title = Z_Strdup("Settings");
	if (!menu->id)
		menu->id = MENU_SETTINGS;

	Menu_AddItem(va("Lock position: %s", (relayflags & RC_LOCKPOS	? "On" : "Off")), "S1", SettingsMenu_Select, RC_LOCKPOS);
	Menu_AddItem(va("Lock view:     %s", (relayflags & RC_LOCKVIEW	? "On" : "Off")), "S1", SettingsMenu_Select, RC_LOCKVIEW);
	Menu_AddItem(va("Chase view:    %s", (relayflags & RC_CHASEVIEW ? "On" : "Off")), "S1", SettingsMenu_Select, RC_CHASEVIEW);
	Menu_AddItem(va("Statusbar:     %s", (relayflags & RC_STATUSBAR ? "On" : "Off")), "S1", SettingsMenu_Select, RC_STATUSBAR);
	Menu_AddItem(va("Layout:        %s", (relayflags & RC_LAYOUT	? "On" : "Off")), "S1", SettingsMenu_Select, RC_LAYOUT);
	Menu_AddItem(va("Tinting:       %s", (relayflags & RC_TINT		? "On" : "Off")), "S1", SettingsMenu_Select, RC_TINT);

	Menu_Finish(menu);
}

//
// Main Menu (static)
//

void MainMenu_Show(menu_t *menu)
{
	Menu_Start(menu);
	if (!menu->title)
		menu->title = Z_Strdup("Main Menu");
	if (!menu->id)
		menu->id = MENU_MAIN;

	Menu_AddItem("Demo info", "S1", Select_OpenMenu, DemoMenu_Show);
	Menu_AddItem("Show Players", "S1", Select_OpenMenu, PlayersMenu_Show);
	Menu_AddItem("Settings", "S1", Select_OpenMenu, SettingsMenu_Show);

	Menu_Finish(menu);
}
